package middleware

import (
	"errors"
	"fmt"
	"git.sch9.ru/new_gate/ms-tester/internal/models"
	"git.sch9.ru/new_gate/ms-tester/internal/sessions"
	"git.sch9.ru/new_gate/ms-tester/pkg"
	"github.com/gofiber/fiber/v2"
	"github.com/golang-jwt/jwt/v4"
	"strings"
)

const (
	TokenKey = "token"
)

func AuthMiddleware(jwtSecret string, sessionsUC sessions.UseCase) fiber.Handler {
	return func(c *fiber.Ctx) error {
		authHeader := c.Get("Authorization", "")
		if authHeader == "" {
			return c.Next()
		}

		authParts := strings.Split(authHeader, " ")
		if len(authParts) != 2 || strings.ToLower(authParts[0]) != "bearer" {
			return c.Next()
		}

		parsedToken, err := jwt.ParseWithClaims(authParts[1], &models.JWT{}, func(token *jwt.Token) (interface{}, error) {
			if _, ok := token.Method.(*jwt.SigningMethodHMAC); !ok {
				return nil, fmt.Errorf("unexpected signing method: %v", token.Header["alg"])
			}

			return []byte(jwtSecret), nil
		})
		if err != nil {
			return c.Next()
		}

		token, ok := parsedToken.Claims.(*models.JWT)
		if !ok {
			return c.Next()
		}

		err = token.Valid()
		if err != nil {
			return c.Next()
		}

		ctx := c.Context()

		// check if session exists
		_, err = sessionsUC.ReadSession(ctx, token.SessionId)
		if err != nil {
			if errors.Is(err, pkg.ErrNotFound) {
				return c.Next()
			}

			return c.SendStatus(pkg.ToREST(err))
		}

		c.Locals(TokenKey, token)
		return c.Next()
	}
}
